@@ -6,12 +6,13 @@ from Algorithmia.algorithm import Algorithm
 from Algorithmia.datafile import DataFile, LocalDataFile, AdvancedDataFile
 from Algorithmia.datadirectory import DataDirectory, LocalDataDirectory, AdvancedDataDirectory
 from algorithmia_api_client import Configuration, DefaultApi, ApiClient
-
+from Algorithmia.util import md5_for_file, md5_for_str
 from tempfile import mkstemp
 import atexit
 import json, re, requests, six, certifi
 import tarfile
 import os
+from time import time
 
 
 class Client(object):
@@ -21,13 +22,20 @@ class Client(object):
     apiKey = None
     apiAddress = None
     requestSession = None
+    bearerToken = None
 
-    def __init__(self, apiKey=None, apiAddress=None, caCert=None):
+
+    def __init__(self, apiKey = None, apiAddress = None, caCert = None, bearerToken=None):
         # Override apiKey with environment variable
         config = None
         self.requestSession = requests.Session()
         if apiKey is None and 'ALGORITHMIA_API_KEY' in os.environ:
             apiKey = os.environ['ALGORITHMIA_API_KEY']
+        if apiKey is None:
+            if bearerToken is None and 'ALGORITHMIA_BEARER_TOKEN' in os.environ:
+                bearerToken = os.environ['ALGORITHMIA_BEARER_TOKEN']
+            self.bearerToken = bearerToken
+
         self.apiKey = apiKey
         if apiAddress is not None:
             self.apiAddress = apiAddress
@@ -217,6 +225,8 @@ class Client(object):
         headers = {}
         if self.apiKey is not None:
             headers['Authorization'] = self.apiKey
+        else: 
+            headers['Authorization'] = "Bearer "+ self.bearerToken
 
         input_json = None
         if input_object is None:
@@ -244,18 +254,24 @@ class Client(object):
         headers = {}
         if self.apiKey is not None:
             headers['Authorization'] = self.apiKey
+        else: 
+            headers['Authorization'] = 'Bearer '+ self.bearerToken
         return self.requestSession.get(self.apiAddress + url, headers=headers, params=query_parameters)
 
     def getStreamHelper(self, url, **query_parameters):
         headers = {}
         if self.apiKey is not None:
             headers['Authorization'] = self.apiKey
+        else: 
+            headers['Authorization'] = 'Bearer '+ self.bearerToken
         return self.requestSession.get(self.apiAddress + url, headers=headers, params=query_parameters, stream=True)
 
     def patchHelper(self, url, params):
         headers = {'content-type': 'application/json'}
         if self.apiKey is not None:
             headers['Authorization'] = self.apiKey
+        else: 
+            headers['Authorization'] = 'Bearer '+ self.bearerToken
         return self.requestSession.patch(self.apiAddress + url, headers=headers, data=json.dumps(params))
 
     # Used internally to get http head result
@@ -263,6 +279,8 @@ class Client(object):
         headers = {}
         if self.apiKey is not None:
             headers['Authorization'] = self.apiKey
+        else: 
+            headers['Authorization'] = 'Bearer '+ self.bearerToken
         return self.requestSession.head(self.apiAddress + url, headers=headers)
 
     # Used internally to http put a file
@@ -270,6 +288,8 @@ class Client(object):
         headers = {}
         if self.apiKey is not None:
             headers['Authorization'] = self.apiKey
+        else: 
+            headers['Authorization'] = 'Bearer '+ self.bearerToken
         if isJson(data):
             headers['Content-Type'] = 'application/json'
 
@@ -283,6 +303,8 @@ class Client(object):
         headers = {}
         if self.apiKey is not None:
             headers['Authorization'] = self.apiKey
+        else: 
+            headers['Authorization'] = 'Bearer '+ self.bearerToken
         response = self.requestSession.delete(self.apiAddress + url, headers=headers)
         if response.reason == "No Content":
             return response
@@ -322,6 +344,30 @@ class Client(object):
         except OSError as e:
             print(e)
 
+    # Used by CI/CD automation for freezing model manifest files, and by the CLI for manual freezing
+    def freeze(self, manifest_path, manifest_output_dir="."):
+        if os.path.exists(manifest_path):
+            with open(manifest_path, 'r') as f:
+                manifest_file = json.load(f)
+            manifest_file['timestamp'] = str(time())
+            required_files = manifest_file['required_files']
+            optional_files = manifest_file['optional_files']
+            for i in range(len(required_files)):
+                uri = required_files[i]['source_uri']
+                local_file = self.file(uri).getFile(as_path=True)
+                md5_checksum = md5_for_file(local_file)
+                required_files[i]['md5_checksum'] = md5_checksum
+            for i in range(len(optional_files)):
+                uri = required_files[i]['source_uri']
+                local_file = self.file(uri).getFile(as_path=True)
+                md5_checksum = md5_for_file(local_file)
+                required_files[i]['md5_checksum'] = md5_checksum
+            lock_md5_checksum = md5_for_str(str(manifest_file))
+            manifest_file['lock_checksum'] = lock_md5_checksum
+            with open(manifest_output_dir+'/'+'model_manifest.json.freeze', 'w') as f:
+                json.dump(manifest_file, f)
+        else:
+            print("Expected to find a model_manifest.json file, none was discovered in working directory")
 
 def isJson(myjson):
     try:
